;; Generated package description from jabber.el  -*- mode: lisp-data; no-byte-compile: t; lexical-binding:t -*-
(define-package "jabber" "0.10.3.0.20260405.232609" "XMPP/Jabber client" '((emacs "29.1") (fsm "0.2.0")) :commit "c303c44003a9f9d5e576c99cc4a302e9cf1142b7" :authors '(("Magnus Henoch" . "mange@freemail.hu")) :maintainer '("Thanos Apollo" . "public@thanosapollo.org") :keywords '("comm") :url "https://git.thanosapollo.org/emacs-jabber")
